#ifndef TTY_HPP
#define TTY_HPP

#include <stdint.h>

#define COLUMNS 80
#define LINES 24

// Some useful #defines
#define BLACK_COLOR 			0
#define BLUE_COLOR 			1
#define GREEN_COLOR 			2
#define CYAN_COLOR 			3
#define RED_COLOR 			4
#define MAGENTA_COLOR 		5
#define BROWN_COLOR 			6
#define LIGHT_GREY_COLOR 		7
#define DARK_GREY_COLOR 		8
#define LIGHT_BLUE_COLOR 		9
#define LIGHT_GREEN_COLOR 		10
#define LIGHT_CYAN_COLOR 		11
#define LIGHT_RED 				12
#define LIGHT_MAGENTA_COLOR 	13
#define LIGHT_BROWN 			14
#define WHITE_COLOR 			15

#define switchToTTY(index) tty = tty ## index
/*
We leave out the last semicolon so we can do this:
	
	switchToTTY(2);
	
which expands to:
	
	tty = tty2;
	
If we added a semicolon in the macro, it would be this:

	tty = tty2;;
*/

class TTY
{
	private:
		const char *name; // The TTY's name
		uint16 *videoMemory; // Location at which to print
		uint8 cursorX;
		uint8 cursorY;
		uint8 backgroundColor;
		uint8 foregroundColor;

		void moveCursor(void);
		void scroll(void);
	
	public:
	
	TTY(void);
	
	TTY(const char *_name);
	
	TTY(const char *_name , uint8 _backgroundColor , uint8 _foregroundColor);
	
	TTY(const char *_name , uint8 _backgroundColor , uint8 _foregroundColor, uint16 *_videoMemory);
	
	void setColor(uint8 _backgroundColor , uint8 _foregroundColor );
	
	bool writef(const char *format , ...);
	
	void write(const char *data);
	
	bool putchar(char ch);
	
	TTY operator<<(const char *data);
	
	TTY operator<<(int data);
	
	TTY operator<<(unsigned int data);
	
	void clear(void);
	
};

extern TTY tty0;
extern TTY tty1;
extern TTY tty2;
extern TTY tty;

#endif