#include <stdint.h>
#include <stdlib.h>
#include <tty.hpp>
#include <port_io.hpp>

#define TAB 0x09

TTY tty0("tty0",BLACK_COLOR,GREEN_COLOR);
// TTY0 is for the kernel; no one else is allowed to use it!
TTY tty1("tty1",BLACK_COLOR,BLUE_COLOR);
// TTY1 is just for drivers
TTY tty2("tty2",BLACK_COLOR,LIGHT_GREY_COLOR);
// TTY2 is for user-mode apps 
TTY tty = tty0;
// Set the default TTY

// Just some simple copy constructors 

TTY::TTY(void) : name("tty0") , cursorX(0) , cursorY(0) , backgroundColor(0) , foregroundColor(7)
{
	videoMemory = (uint16 *)0xB8000;
}

TTY::TTY(const char *_name) : name(_name) , foregroundColor(7)
{
	videoMemory = (uint16 *)0xB8000;
}

TTY::TTY(const char *_name , uint8 _backgroundColor , uint8 _foregroundColor) : name(_name) , cursorX(0) , cursorY(0) , backgroundColor(_backgroundColor) , foregroundColor(_foregroundColor)
{
	videoMemory = (uint16 *)0xB8000;
}

TTY::TTY(const char *_name, uint8 _backgroundColor , uint8 _foregroundColor , uint16 * _videoMemory) : name(_name) , videoMemory(_videoMemory) , cursorX(0) , cursorY(0) , backgroundColor(_backgroundColor) , foregroundColor(_foregroundColor)
{
}

void TTY::moveCursor(void)
{
	uint16 cursorLocation = cursorY * COLUMNS + cursorX;
	
	if(videoMemory == (uint16 *)0XB8000) // We are the actual screen
	{
		outb(0x3D4 , 14);
		outb(0x3D5 , cursorLocation >> 8);
		outb(0x3D4 , 15);
		outb(0x3D5 , cursorLocation);
	}
	else // We are a virtual terminal
	{
		cursorX++;
		
		if(cursorX >= COLUMNS)
		{
			cursorY++;
			cursorX = 0;
		}
	}
}

void TTY::setColor(uint8 _backgroundColor , uint8 _foregroundColor)
{
	backgroundColor = _backgroundColor;
	foregroundColor = _foregroundColor;
}

bool TTY::writef(const char *format, ...)
{
	format = 0;
	return true;
}

bool TTY::putchar(char ch)
{
	uint8 attributeByte = (backgroundColor << 4) | (foregroundColor & 0x0F );
	uint16 attribute = attributeByte << 8;
	uint16 *location;
	
	if (ch == 0x08 && cursorX) // Backspace?
		cursorX--;
	
	else if (ch == TAB) // Tab?
		cursorX = ( cursorX + 8 ) & ~( 8 - 1 );
	
	else if (ch == '\r') // Carriage return?
		cursorX = 0;
	
	else if (ch == '\n') // Newline?
	{
		cursorX = 0;
		cursorY++;
	}
	
	else if (ch >= ' ') // I guess it's none of those other ones..
	{
		location = videoMemory + ( cursorY * COLUMNS + cursorX);
		*location = ch | attribute; // Print it!
		cursorX++;
	}
	
	return true;
}
 
void TTY::clear(void)
{
	// This is pretty self-explanatory, so i won't go into detail
	
	uint8 attribute = (backgroundColor << 4) | ( foregroundColor & 0x0F);
	uint16 blank = 0x20 | ( attribute << 8 );
	
	for(int i = 0; i < COLUMNS * LINES; i++)
	videoMemory[i] = blank;
	
	cursorX = 0;
	cursorY = 0;
	
	moveCursor();
}

void TTY::scroll(void)
{
	// Same here
	uint8 attribute = ( backgroundColor << 4 ) | ( foregroundColor & 0x0F );
	uint16 blank = 0x20 | (attribute << 8 );
	
	if(cursorY >= 25)
	{
		for(int i = 0 * COLUMNS; i < LINES * COLUMNS; i++)
		videoMemory[i] = videoMemory[i + 80];
	}
	
	for(int i = 24 * COLUMNS; i < LINES * COLUMNS; i++)
	videoMemory[i] = blank;
	
	cursorY = 24;
}

void TTY::write(const char *data)
{
	int i = 0;
	
	while(data[i]) // While there's still something to print
		putchar(data[i++]); // Print it!
}

TTY TTY::operator<<(const char *data)
{
	int i = 0;
	
	while(data[i]) // While there's still something to print
		putchar(data[i++]); // Print it!
	
	return *this; // We return *this so we can do this(pun intended!):
	/*
	*	tty << "One string.." << "Two strings!";
	*	
	*	instead of:
	*
	*	tty << "One string..";
	*	tty << "Two strings!";
	*
	*
	*	See how much nicer that looks?
	*	See the note on memmove() in stdlib.h
	*/
}

TTY TTY::operator<<(int data) // Just a simple wrapper for write
{
	char buffer[30];
	
	itoa(buffer, 'd', data);
	
	write(buffer);
	
	return *this;
}

TTY TTY::operator<<(unsigned int data) // Same thing, just for unsigned ints
{
	char buffer[30];
	
	itoa(buffer, 'd', data);
	
	write(buffer);
	
	return *this;
}